import { useMemo } from 'react';
import { scaleLinear } from 'd3-scale';
import { maxIndex } from 'd3-array';
import { DEFAULT_MAX_Y } from '../constants.js';
import { useChartContext } from './useChartContext.js';
import { isInfinity } from '../utilities/isInfinity.js';
import { shouldRoundScaleUp } from '../utilities/shouldRoundScaleUp.js';
import { estimateStringWidth } from '../utilities/estimateStringWidth.js';

const MINIMAL_LABEL_SPACE = 80;
function useYScale({
  drawableHeight,
  formatYAxisLabel,
  integersOnly = false,
  max,
  min,
  shouldRoundUp = true,
  verticalOverflow = true,
  fixedWidth,
  maxYOverride,
  ticksOverride
}) {
  const {
    characterWidths
  } = useChartContext();
  if (maxYOverride != null && maxYOverride < 0) {
    throw new Error('maxYOverride must be a non-negative number.');
  }
  const [minY, maxY] = useMemo(() => {
    const isDataEmpty = min === 0 && max === 0;
    const minY = min;
    let maxY = isDataEmpty ? DEFAULT_MAX_Y : max;
    if (maxYOverride != null && isDataEmpty) {
      maxY = maxYOverride;
    }
    if (integersOnly) {
      return [Math.floor(minY), Math.ceil(maxY)];
    }
    return [minY, maxY];
  }, [min, max, integersOnly, maxYOverride]);
  if (isInfinity(maxY) || isInfinity(minY)) {
    throw new Error('Data must be finite.');
  }
  const {
    yScale,
    ticks,
    yAxisLabelWidth
  } = useMemo(() => {
    const maxTicks = Math.max(1, Math.ceil(drawableHeight / MINIMAL_LABEL_SPACE));
    const yScale = scaleLinear().range([drawableHeight, 0]).domain([Math.min(0, minY), Math.max(0, maxY)]);

    // if verticalOverflow is false, always round up
    // if verticalOverflow is true, only round up if both shouldRoundUp and shouldRoundScaleUp is true
    if (!verticalOverflow || shouldRoundUp && shouldRoundScaleUp({
      yScale,
      maxValue: maxY,
      maxTicks
    })) {
      yScale.nice(maxTicks);
    } else {
      const roundedDownMin = yScale.copy().nice(maxTicks).ticks(maxTicks)[0];
      yScale.domain([Math.min(roundedDownMin, minY), Math.max(0, maxY)]);
    }
    const filteredTicks = getFilteredTicks();
    const ticks = filteredTicks.map(value => ({
      value,
      formattedValue: formatYAxisLabel(value),
      yOffset: yScale(value)
    }));
    const longestYAxisLabel = maxIndex(ticks, ({
      formattedValue
    }) => formattedValue == null ? 0 : formattedValue.length);
    const text = ticks[longestYAxisLabel] ? ticks[longestYAxisLabel].formattedValue : '';
    const yAxisLabelWidth = estimateStringWidth(text, characterWidths);
    return {
      yScale,
      ticks,
      yAxisLabelWidth
    };
    function getFilteredTicks() {
      if (ticksOverride != null) {
        return ticksOverride;
      }
      return integersOnly ? yScale.ticks(maxTicks).filter(tick => Number.isInteger(tick)) : yScale.ticks(maxTicks);
    }
  }, [verticalOverflow, shouldRoundUp, characterWidths, drawableHeight, formatYAxisLabel, integersOnly, maxY, minY, ticksOverride]);
  return {
    yScale,
    ticks,
    yAxisLabelWidth: getLabelWidth(yAxisLabelWidth, fixedWidth)
  };
}
function getLabelWidth(yAxisLabelWidth, fixedWidth) {
  if (fixedWidth === false || fixedWidth == null) {
    return yAxisLabelWidth;
  }
  return fixedWidth;
}

export { getLabelWidth, useYScale };
